<?php

namespace Drupal\emailoctopus\Plugin;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use Drupal\emailoctopus\Exception\InvalidApiKeyException;
use Drupal\emailoctopus\Exception\InvalidParametersException;
use Drupal\emailoctopus\Exception\ResourceNotFoundException;
use Drupal\emailoctopus\Exception\UnauthorisedException;
use Drupal\emailoctopus\Exception\MemberExistsWithEmailAddressException;
use Psr\Http\Message\ResponseInterface;
use Drupal\emailoctopus\ApiInterface;


/**
 * Class ApiClient
 *
 * @package Ideneal\EmailOctopus\Http
 */
class ApiClient implements ApiInterface
{
    const BASE_URL    = 'https://emailoctopus.com/api';
    const API_VERSION = '1.6';

    /**
     * @var string
     */
    private $apiKey;

    /**
     * @var Client
     */
    private $client;

    /**
     * ApiClient constructor.
     *
     * @param string $apiKey
     * @param array  $config
     */
    public function __construct(string $apiKey, array $config = [])
    {
        $this->apiKey = $apiKey;

        $this->client = new Client(array_merge([
            'base_uri'    => self::BASE_URL . '/' . self::API_VERSION . '/',
            'http_errors' => false,
        ], $config));
    }

    /**
     * @param string $path
     * @param array  $params
     *
     * @return ResponseInterface
     * @throws InvalidApiKeyException
     * @throws InvalidParametersException
     * @throws ResourceNotFoundException
     * @throws UnauthorisedException
     * @throws GuzzleException
     */
    public function get(string $path, array $params = []): ResponseInterface
    {
        $response = $this->client->get($path, [
            'query' => $this->processQuery($params),
        ]);

        $this->validateResponse($response);

        return $response;
    }

    /**
     * @param string $path
     * @param array  $data
     * @param array  $params
     *
     * @return ResponseInterface
     * @throws InvalidApiKeyException
     * @throws InvalidParametersException
     * @throws ResourceNotFoundException
     * @throws UnauthorisedException
     * @throws GuzzleException
     */
    public function post(string $path, array $data = [], array $params = []): ResponseInterface
    {
        $response = $this->client->post($path, [
            'query' => $this->processQuery($params),
            'json'  => $data,
        ]);

        $this->validateResponse($response);

        return $response;
    }

    /**
     * @param string $path
     * @param array  $data
     * @param array  $params
     *
     * @return ResponseInterface
     * @throws InvalidApiKeyException
     * @throws InvalidParametersException
     * @throws ResourceNotFoundException
     * @throws UnauthorisedException
     * @throws GuzzleException
     */
    public function put(string $path, array $data = [], array $params = []): ResponseInterface
    {
        $response = $this->client->put($path, [
            'query' => $this->processQuery($params),
            'json'  => $data,
        ]);

        $this->validateResponse($response);

        return $response;
    }

    /**
     * @param string $path
     * @param array  $params
     *
     * @return ResponseInterface
     * @throws InvalidApiKeyException
     * @throws InvalidParametersException
     * @throws ResourceNotFoundException
     * @throws UnauthorisedException
     * @throws GuzzleException
     */
    public function delete(string $path, array $params = []): ResponseInterface
    {
        $response = $this->client->post($path, [
            'query' => $this->processQuery($params),
        ]);

        $this->validateResponse($response);

        return $response;
    }

    /**
     * Process the request query before send it.
     *
     * @param array $params
     *
     * @return array
     */
    private function processQuery(array $params = []): array
    {
        return array_merge($params, [
            'api_key' => $this->apiKey,
        ]);
    }

    /**
     * Validates the response.
     *
     * @param ResponseInterface $response
     *
     * @throws InvalidApiKeyException
     * @throws InvalidParametersException
     * @throws ResourceNotFoundException
     * @throws UnauthorisedException
     * @throws \Exception
     */
    private function validateResponse(ResponseInterface $response)
    {

        if ($response->getStatusCode() !== 200) {
            $json    = json_decode($response->getBody(), true);
            $error   = $json['error'];
            $code    = $error['code'];
            $message = $error['message'];

            \Drupal::messenger()->addMessage($message, 'error');

            switch ($code) {
                case 'INVALID_PARAMETERS':
                throw new InvalidParametersException($message);

                case 'API_KEY_INVALID':
                throw new InvalidApiKeyException($message);

                case 'UNAUTHORISED':
                throw new UnauthorisedException($message);

                case 'NOT_FOUND':
                throw new ResourceNotFoundException($message);

                case 'MEMBER_EXISTS_WITH_EMAIL_ADDRESS':
                throw new MemberExistsWithEmailAddressException($message);

                default:
                throw new \Exception($message);
            }
        }
    }
}